#include <cmath>
#include <cstdlib>
#include <fstream>
#include <iostream>
#include <string>
using namespace std;

const double g = 9.80;           // acceleration of gravity
const double L = 1;              // length in meters

struct Point {                   // object to store dynamical variables
    double t;
    double theta;
    double omega;
};

void EulerStep(                   // trajectory point after one Euler time step
    Point& p,                     // trajectory point - input/output variable
    double dt)                    // time step for Euler - input variable
{
    double t = p.t, theta = p.theta, omega = p.omega;
    double omegaOld = omega;

    // apply Euler's algorithm
    omega -= (g/L) * theta * dt;
    theta += omegaOld * dt;
    t += dt;

    p.t = t, p.theta = theta, p.omega = omega;
}

void EulerCromerStep(             // trajectory point after Euler-Cromer step
    Point& p,                     // trajectory point - input/output variable
    double dt)                    // time step for Euler - input variable
{
    double t = p.t, theta = p.theta, omega = p.omega;

    // apply Euler-Cromer algorithm
    omega -= (g/L) * theta * dt;
    theta += omega * dt;
    t += dt;

    p.t = t, p.theta = theta, p.omega = omega;
}

void simulate(                   // generate trajectory
    void method(Point&, double), // name of time step function
    double tMax,                 // total integration time
    double dt,                   // time step
    string fileName)             // output data file name
{
    // initialize
    Point p;
    p.t = 0;
    p.theta = 0;
    p.omega = 1;
    ofstream file(fileName.c_str());

    // algorithm loop
    while (p.t <= tMax) {
        method(p, dt);
        file << p.t << '\t' << p.theta << '\t' << p.omega << '\n';
    }

    // finalize
    cout << "Trajectory data in file " << fileName << endl;
    file.close();
}

int main() {
    cout << "Simulation of Simple Pendulum using Euler & Euler-Cromer\n"
         << "--------------------------------------------------------\n";
    double tMax, dt;
    cout << "Enter time step dt (s): ";
    cin >> dt;
    cout << "Enter integration time t_max: ";
    cin >> tMax;

    cout << "Running simulation using Euler method" << endl;
    simulate(EulerStep, tMax, dt, "euler.data");
    cout << "Running simulation using Euler-Cromer method" << endl;
    simulate(EulerCromerStep, tMax, dt, "euler-cromer.data");
}
